#uses "classes/unHMI/UnHMI_ScreenLayout.ctl"

private const string DEFAULT_SCREENLAYOUT_DP = "_unApplication.screenLayout";
private const string VALIDATION_HEADER = "SCREENLAYOUT VALIDATION HEADER";
private bool gImportYesToAll = true;


class UnHMI_ScreenLayoutHelper
{
  private vector< shared_ptr<UnHMI_ScreenLayout> > screenLayouts;

  private static shared_ptr<UnHMI_ScreenLayoutHelper> singleton_instance = nullptr;

  public static string UnHMI_ScreenLayout_FILTER_PREFIX;

  /*private constructor - Use getInstance - Singleton pattern */
  private UnHMI_ScreenLayoutHelper()
  {
    this.loadFromDatapoint();
  }

  public static synchronized shared_ptr<UnHMI_ScreenLayoutHelper>  getInstance()
  {
    if(equalPtr(singleton_instance, nullptr))
    {
      singleton_instance = new UnHMI_ScreenLayoutHelper();
    }
    return singleton_instance;
  }

  synchronized public dyn_string getScreenLayoutNames(bool disableAC = false)
  {
    dyn_string screenLayoutNames;
    int cnt = screenLayouts.count();
    for (int i=0; i< cnt; i++)
    {
      shared_ptr<UnHMI_ScreenLayout> tempPtr = screenLayouts.at(i);
      dyn_string exceptionInfo;
      bool granted;
      fwAccessControl_isGranted(tempPtr.getAccessControl(), granted, exceptionInfo);
      if (granted || tempPtr.getAccessControl() == "" || disableAC)
      {
        screenLayoutNames.append(tempPtr.getScreenLayoutName());
      }
    }
    return screenLayoutNames;
  }

  public dyn_string getDefaultScreenLayoutNames()
  {
    string json;
    mapping defaultScreenLayouts;
    dyn_string dsRet;

    dpGet(DEFAULT_SCREENLAYOUT_DP,  json);
    if (json.length() > 0)
    {
      fwJsonDeserialize(json, defaultScreenLayouts, makeMapping("acceptIncompleteJson", true));
      dyn_string dsKeys = mappingKeys(defaultScreenLayouts);
      for (int i = 1; i <= dynlen(dsKeys); i++)
      {
        dsRet.append(defaultScreenLayouts[dsKeys[i]]);
      }
    }
    return dsRet;
  }

  public string getDefaultScreenLayout(string frontEndApplication = "")
  {
    mapping defaultScreenLayouts;
    string json, ret;
    if (frontEndApplication == "" )
    {
      frontEndApplication = "_DEFAULT";
    }

    dpGet(DEFAULT_SCREENLAYOUT_DP,  json);
    if (json.length() <1) return "";

    fwJsonDeserialize(json, defaultScreenLayouts, makeMapping("acceptIncompleteJson", true));
    if (mappingHasKey(defaultScreenLayouts, frontEndApplication))
    {
      ret = defaultScreenLayouts[frontEndApplication];
    } else
    {
      ret = "";
    }
    return ret;
  }

  public void setDefaultScreenLayout(string frontEndApplication, string defaultScreenLayoutName)
  {
    mapping defaultScreenLayouts;
    string json;
    if (frontEndApplication == "" ) {
      frontEndApplication = "_DEFAULT";
    }

    dpGet(DEFAULT_SCREENLAYOUT_DP,  json);
    if (json.length() > 0){
      fwJsonDeserialize(json, defaultScreenLayouts, makeMapping("acceptIncompleteJson", true));
    }
    defaultScreenLayouts[frontEndApplication] = defaultScreenLayoutName;

    json = fwJsonSerialize(defaultScreenLayouts);
    dpSet(DEFAULT_SCREENLAYOUT_DP, json);

  }

  public shared_ptr<UnHMI_ScreenLayout> getScreenLayout(string screenLayoutName)
  {
    shared_ptr<UnHMI_ScreenLayout> retScreenLayout;

    // ScreenLayoutName is used as a key, so no duplicates exists
    vector<int> idx = screenLayouts.indexListOf("screenLayoutName", screenLayoutName);

    if (idx.isEmpty()) {
      retScreenLayout = nullptr;
    } else {
      retScreenLayout = screenLayouts.at(idx.first());
    }
    return retScreenLayout;
  }

  public void setScreenLayout(shared_ptr<UnHMI_ScreenLayout> sl)
  {
    vector<int> idx = screenLayouts.indexListOf("screenLayoutName", sl.getScreenLayoutName());

    // Check if already exists
    if (!idx.isEmpty())
    {
      // Overwrite it if it does
      screenLayouts.replaceAt(idx.first(), sl);
    } else {
      // Otherwise append
      screenLayouts.append(sl);
    }
  }

  synchronized public void deleteScreenLayout(string screenLayoutName)
  {
    vector<int> idx = screenLayouts.indexListOf("screenLayoutName", screenLayoutName);
    if (idx.isEmpty()) {
      DebugFTN("unHMI", "ERROR. The screen layout selected cannot be deleted because it does not exist or it cannot be found");
    }
    shared_ptr<UnHMI_ScreenLayout> sl = screenLayouts.takeAt(idx.first());
    sl.deleteDatapoint();

    // Extra handling delete from default list
    dyn_string split = strsplit(screenLayoutName, "/");

    // Check if frontEndApplication
    if ( (dynlen(split)==2) && (screenLayoutName == getDefaultScreenLayout(split[1])) )
    {
      setDefaultScreenLayout(split[1], "");
    } else if ( (dynlen(split) == 1) && (screenLayoutName == getDefaultScreenLayout()) )
    {
      setDefaultScreenLayout("", "");
    }
  }

  synchronized public void saveToDatapoint(string screenLayoutName)
  {
    vector<int> idx = screenLayouts.indexListOf("screenLayoutName", screenLayoutName);
    if (idx.isEmpty()) {
      DebugFTN("unHMI", "ERROR. The screen layout selected cannot be saved because it does not exist or it cannot be found");
    }
    shared_ptr<UnHMI_ScreenLayout> sl = screenLayouts.at(idx.first());
    sl.saveToDatapoint();
  }

  synchronized public void loadFromDatapoint(string accessControl = "")
  {

    dyn_string dsDpNames = getFilteredScreenLayoutDpNames();
    screenLayouts.clear();
    for (int i=1; i<= dsDpNames.count(); i++)
    {
      shared_ptr<UnHMI_ScreenLayout> tempPtr = UnHMI_ScreenLayout::createWithDpName(dsDpNames[i]);
      screenLayouts.append(tempPtr);
    }
  }

  private dyn_string getFilteredScreenLayoutDpNames(){
    string selectQuery = "SELECT '_online.._value' FROM '*' WHERE _DPT = \"" + "UnHMI_ScreenLayout" + "\"" +
                                                             " && _DP LIKE \""+ "UnHMI_ScreenLayout/" + UnHMI_ScreenLayout_FILTER_PREFIX +"*\"";
    dyn_string dsDpNames;
    unFilterDpFunctions_dpQuery(selectQuery, dsDpNames);
    return dsDpNames;
  }

  public bool screenLayoutExists(string screenLayoutName)
  {
    shared_ptr<UnHMI_ScreenLayout> sl = singleton_instance.getScreenLayout(screenLayoutName);
    if (equalPtr(sl, nullptr))
    {
       return false;
    } else
    {
      return true;
    }
  }

  //Loading screenlayout on demand
  public void loadScreenLayout(string screenLayoutName)
  {
    shared_ptr<UnHMI_ScreenLayout> sl = singleton_instance.getScreenLayout(screenLayoutName);

	if (equalPtr(sl, nullptr)) {
		DebugFTN("unHMI", "ERROR. The screen layout selected cannot be loaded because it does not exist or it cannot be found");
		return;
	}
	sl.loadFromDatapoint();

    vector< shared_ptr<UnHMI_Window> > windows = sl.getWindows();
    shape mainWindow = getShape(UNHMI_MAIN_MODULE_NAME + "." + UNHMI_MAIN_PANEL_NAME +":");
	for (int i=0; i<windows.count(); i++)
    {
      UnHMI_Window win =  windows.at(i);
      string fileName = win.getFileName();

      string datapoint = win.getDatapoint();
      string sDollarParam, sDpName;
      if (!datapoint.isEmpty())
      {
        string sDpPanel, sDpType;
        dyn_string dsUserData, exceptionInfo;
        if(dpExists(datapoint) && (dpTypeName(datapoint) == unTreeWidget_treeNodeDPT))
        {
		  sDpName=sDpPanel=""; // reset
		  fwTree_getNodeUserData(_fwTree_getNodeName(datapoint), dsUserData, exceptionInfo);
		  fwTree_getNodeDevice(_fwTree_getNodeName(datapoint), sDpPanel, sDpType, exceptionInfo);
		  int len = dynlen(dsUserData);
		  if(len > 0) sDollarParam = dsUserData[1];
			if(len <= c_unPanelConfiguration_maxPanelNavigation) sDpName = sDpPanel;
			else sDpName = datapoint;
		}
	  }

      if (!fileName.isEmpty())
      {
        if (!mainWindow.unHMI_getIsPreviewModuleOpen(win.getNumber()))
        {
          mainWindow.unHMI_newPreviewModule(fileName, win.getNumber());
        }
        unHMI_openPreviewModuleHelper(fileName, win.getNumber(), sDollarParam, sDpName);
      } else
      {
        mainWindow.unHMI_removePreviewModule(win.getNumber());
      }
    }
  }
  /**
   *
   * @return status: 1 for success, -1 for error,  0 for empty file
   */
  public int exportScreenLayoutConfiguration(dyn_string &exceptionInfo, string fName, string &exportedScreenLayoutNames)
  {
    dyn_string dsDpNames = getFilteredScreenLayoutDpNames();
    int cnt = dsDpNames.count();
    file fileToExport = fopen(fName, "w");

    if (fileToExport==0)
    {
      dynAppend(exceptionInfo, "Could not open file for writing");
      return -1;
    }

    // Empty file
    if (cnt < 1) return 0;

    //Validation Header
    fputs(VALIDATION_HEADER + "\n", fileToExport);

    for (int i = 1; i <= cnt; i++)
    {
      string json, accessControl;
      string dpName = dsDpNames[i];
      dyn_string split = strsplit(dpName, ":");
      dpGet(dpName + ".screenLayout", json);
      dpGet(dpName + ".accessControl", accessControl);

      // dpName = dist%d + : + actualdpName , so we only keep the actualdpName
      fputs(split[2] + "\n", fileToExport);
      fputs(json + "\n", fileToExport);
      fputs(accessControl + "\n", fileToExport);

      //dpName = "UnHMI_ScreenLayout/" + screenlayoutname, we need to remove the prefix
      strreplace(split[2], "UnHMI_ScreenLayout/", "");
      exportedScreenLayoutNames = exportedScreenLayoutNames + split[2] + "|";
    }
    fclose(fileToExport);
    return 1;
  }

  /**
   *
   * @return status: 1 for success, -1 for error
   */
  public int importScreenLayoutConfiguration(dyn_string &exceptionInfo, string fName, string &importedScreenLayoutNames)
  {
    int i = access(fName, F_OK);
    //File does not exist
    if ( i!= 0) return -1;

    file fileToImport = fopen(fName, "r");
    string sLine, dpName, json, accessControl;
    fgets(sLine, strlen(VALIDATION_HEADER), fileToImport);
    if (sLine != VALIDATION_HEADER)
    {
      fwException_raise(exceptionInfo, "ERROR", __FUNCTION__ + ": Can not import file " + fName + ".\n Not a valid screenlayout importation file", "");
      return -1;
    }

    gImportYesToAll = false;
    int overwriteAction;
    while( feof(fileToImport) == 0)
    {
      // DpName
      fgets(sLine, 100000, fileToImport);
      dpName = strrtrim(sLine, "\n");
      if (dpName == "")
      {
        continue;
      }

      string slName = substr(dpName, strpos(dpName, "/")+1);

      //reset
      overwriteAction = 1;
      if (!gImportYesToAll && dpExists(dpName)) {
        overwriteAction = _overwriteScreenLayoutDialog(slName);
      }

      if (overwriteAction == - 1) break;
      if (overwriteAction == 0) {
        //Dump next two lines
        fgets(sLine, 100000, fileToImport);
        fgets(sLine, 100000, fileToImport);
        continue;
      }
      importedScreenLayoutNames = importedScreenLayoutNames + slName + "|";

      dpCreate(dpName, "UnHMI_ScreenLayout");
      // SL serialized
      fgets(sLine, 100000, fileToImport);
      json = strrtrim(sLine, "\n");
      dpSet(dpName + ".screenLayout", json);
      // AC
      fgets(sLine, 100000, fileToImport);
      accessControl = strrtrim(sLine, "\n");
      dpSet(dpName + ".accessControl", accessControl);
    }
    fclose(fileToImport);
    return 1;
  }
};

private int _overwriteScreenLayoutDialog(string slName)
{
  dyn_string ds;
  dyn_float df;
  int ret;
  ChildPanelOnCentralReturn("fwInstallation/fwInstallation_messageInfo3", "Warning", makeDynString("$1:ScreenLayout \"" + slName  + "\" already exists.\n\nDo you want to overwrite it?"), df, ds);
  switch(df[1]){
    // Cancel
    case -1:
      ret = -1;
      break;
    // No
    case 0:
      ret = 0;
      break;
    // Yes
    case 1:
      ret = 1;
      break;
    // Yes to all
    case 2:
      ret =1;
      gImportYesToAll = true;
      break;
    default:
      break;
  }
  return ret;
}
