#uses "CtrlPv2Admin"

class  UnModbusRedirectEntry {

    public string plcIP;   // CFG_IP
    public int    port ;   // CFG_PORT
    public string project; // CFG_PROJ
    public int    driver;  // CFG_DRIVER
    public string comment; // CFG_COMMENT
    public bool   conf_inFile;// CFG_CONFRED
    public bool   conf_live;// CFG_CONFMOD
    public string hostname;// CFG_HOSTNAME

    public UnModbusRedirectEntry(
	string newPlcIP="", int newPort=0, string newProject="", int newDriver=0,
	string newComment="", bool   newConfInFile=false, bool   newConfLive=false, string newHostname="") 
    {
		plcIP    = newPlcIP;
		port     = newPort;
		project  = newProject;
		driver   = newDriver;
		comment  = newComment;
		conf_inFile = newConfInFile;
		conf_live = newConfLive;
		hostname = newHostname;
    }
    public string serialize() 
    {	
    	return jsonEncode(this,true); // 2nd param means compact version
    }

    public static UnModbusRedirectEntry deserialize(string s)
    {
		mapping m=jsonDecode(s);
		dyn_errClass err=getLastError();
		if (dynlen(err)) {
			throwError(makeError("unModbusRedirect", PRIO_WARNING, ERR_CONTROL, 2000,"Problem with decoding data entry: "+s));
    		UnModbusRedirectEntry e=UnModbusRedirectEntry();
    		e.comment="DECODING ERROR:"+s;
    		e.plcIP="#";
    		return e;
		}
    	UnModbusRedirectEntry e=UnModbusRedirectEntry(m["plcIP"],m["port"],m["project"],m["driver"],m["comment"],m["conf_inFile"],m["conf_live"],m["hostname"]);
    	return e;
    }
};

class UnModbusRedirectConfiguration {

	public string error="";
	protected dyn_mixed entries;


	/* Print formatted information to the log
	*/
	void INFO(string message, int prio=PRIO_INFO, string extraInfo="")
	{
		if (extraInfo=="") {
			throwError(makeError("unModbusRedirect", PRIO_INFO, ERR_CONTROL, 2000, message));
		} else {
			throwError(makeError("unModbusRedirect", PRIO_INFO, ERR_CONTROL, 2000, message, extraInfo));
		}
	}

	/* Set an error and print to the log
	
		Sets the internal error variable, and prints it to the log 
		together with additional diagnostic information.
		
		Typically diagnostic info should be a string, which is then printed
		as the part of the log message. However, if it is of dyn_errClass type,
		then instead the errorText is extracted from it.
		e.g.
		```		
		if (paCfgReadValueList(...)) {
			ERROR("COULD NOT GET INFO",getLastError());return;
		}
		```
		Or, with some more processing
		```		
		paCfgReadValueList(...);
		dyn_errClass err=getLastError();
		if (getErrorCode(err)!=46) { ERROR("COULD NOT GET INFO",err);return;}
		```
		
		The return value from the function is not important - it is always -1.
		Still, it allows for a handy code snippet in functions that by convention
		return a non-zero (eg. -1) value in the event of an error with the
		call to ERROR() combined with return:
		```
			int rc=Something();
			if (rc) return ERROR("Error in call to something");
			
		```
	*/
	int ERROR(string message, mixed diagnosticInfo="")
	{
		string diag = diagnosticInfo;
		
		if (getType(diagnosticInfo)==DYN_ERRCLASS_VAR) {
			if (dynlen(err)) diag=getErrorText(err);
		}
		this.error=message;
		if (diag=="") {
			throwError(makeError("unModbusRedirect", PRIO_SEVERE, ERR_CONTROL, 2000, message));
			return -1;
		} else {
			throwError(makeError("unModbusRedirect", PRIO_SEVERE, ERR_CONTROL, 2000, message, diag));
			return -1;
		}
		return -1;
	}

    public UnModbusRedirectConfiguration()
    {
    }

	public static UnModbusRedirectConfiguration fromFile(string cfgFile)
	{

		UnModbusRedirectConfiguration cfg;
		dynClear(cfg.entries);

		if (cfgFile=="") return cfg;
		
		string s;
		bool ok=fileToString(cfgFile,s); // note! getLastError() does not work here; error is thrown to the log anyway - no way to catch it
		if (!ok) {
			//cfg.error="ERROR while reading the file:"+cfgFile;
			cfg.ERROR("ERROR while reading the file:"+cfgFile);
			return cfg;
		}
		
		dyn_string ds=strsplit(s,"\n");
		for (int i=1;i<=dynlen(ds);i++) {
			if (ds[i]=="") continue;//empty line
			if (substr(ds[i],0,1)=="#") {
				// a comment; keep a special entry that has "#" as PLC IP
				string comment=strltrim(ds[i],"# ");
				UnModbusRedirectEntry e=UnModbusRedirectEntry("#",0,"",0,comment,true,false,getHostname());
				dynAppend(cfg.entries,e);
				continue;
			}

			dyn_string tokens;
			//int rc=regexpSplit("^\\s*(\\S+)\\s+(\\S+)\\s+(\\d+)\\s+(\\S+)\\s+(\\#*)\\s*(.+)",ds[i],tokens);
			int rc=regexpSplit("^\\s*(\\S+)\\s+(\\S+)\\s*(\\S*)\\s*(\\S*)\\s*(#*)(\\s*)(.+)?",ds[i],tokens);
			
			// Note that tokens[1] matches the complete string, hence we get captured 
			// substring starting at index 2
			//
			// The regexp above works as follows
			// ^ start of the string
			// \s* any amount of white space, or no white space
			// (\S+) PLC Address string => captured into tokens[2]
			// \s+   whitespace (at least one)
			// (\S+) Port number => captured into tokens[3]
			// \s*   whitespace (optional)
			// (\S*) Project name (optional) => captured into tokens[4]
			// \s*   whitespace (optional)
			// (\S*) driver number (optional) => captured into tokens[5]
			// \s*   whitespace (optional)
			// (#*)  #-separator of the comment (optional) => captured into tokens[6]
			// \s*   whitespace that optionally follows the "#" => captured into tokens[7]
			// (.+)? comment (optional), any number of words (also none) => captured into tokens[8]
			
			if (rc == -1) {
				// could not parse the file; $1 is line number, $2 is the entry
			  	throwError(makeError("unModbusRedirect", PRIO_WARNING, ERR_CONTROL, 2001,(string)i ,ds[i]));
			  	continue;
			}
			
			UnModbusRedirectEntry e=UnModbusRedirectEntry(
				tokens[2],      //plc address
				(int)tokens[3], //port number
				tokens[4],      //project
				(int)tokens[5], //driver number
				tokens[8],      //comment
				true,			//conf_inFile
				false,			//conf_live
				getHostname()	//host
				);
			dynAppend(cfg.entries,e);
		}
		
		cfg.error="";
		return cfg;
	}

	public int numEntries()
	{
		return dynlen(entries);
	}
	
	/**
		Note: zero-based indexing!
	*/
	public UnModbusRedirectEntry at(unsigned i)
	{
		UnModbusRedirectEntry e=entries[i+1];
		return e;
	}


	public void clear()
	{
		dynClear(entries);
	}
	
	public void addEntry(UnModbusRedirectEntry e)
	{
		dynAppend(entries,e);
	}

    public dyn_mixed getConfiguration() {
    	if (error!="") return makeDynString();
		return entries;
    }

    public void saveToFile(string cfgFile)
    {
    	string newContent;

    	bool ok;
    	// check if file exists, make a backup, preserve headers
    	int rc=access(cfgFile,R_OK);
    	if (rc) {
    		// file does not exist... create one from template
			string templCfgFile=getPath(DATA_REL_PATH,"unModbusRedirect/redirectModbus.cfg.template");
        	if (templCfgFile=="") {ERROR("Template config file does not exist");return;}
        	ok=copyFile(templCfgFile,cfgFile);
        	if (!ok) {ERROR("Could not create config file from template",getLastError());return;}
        }

        ok=copyFile(cfgFile, cfgFile+".bak");
        if (!ok) {ERROR("Backup of config file failed",getLastError());return;}
        
        string s;
        bool ok=fileToString(cfgFile,s);
        if (!ok) {ERROR("Could not open config file for reading",getLastError());return;}
        

		// and then iterate on what we have
		for (int i=1;i<=dynlen(entries);i++) {
			UnModbusRedirectEntry e=entries[i];
			if (i!=1) newContent+="\n"; // end the previous line
			// special treatment for comment lines:
			if (e.plcIP=="#") {
				newContent+="# "+e.comment;
				continue;
			}
			newContent+=e.plcIP;
			newContent+="\t"+e.port;
			if (e.project=="") continue;
			newContent+="\t"+e.project;
			newContent+="\t"+e.driver;
			if (e.comment=="") continue;
			newContent+="\t# "+e.comment;
		}
		if (dynlen(entries)>0) newContent+="\n"; // append newline at the last line, if there are any entries
		file f=fopen(cfgFile,"w");
		if (f==0) {ERROR("Could not open config file for writing");return;}
		int rc=fputs(newContent,f);
		if (rc<0) {ERROR("Could not write to config file",getLastError());return;}
		fclose(f);

		rc=handleProjectConfigFile(); // it sets the .error field already
    }

	public int handleProjectConfigFile()
	{
		int rc=0;

		string configFilePath=getPath(CONFIG_REL_PATH,"config");

		mapping newDrvPorts, rmDrvPorts;

		for (int i=0;i<entries.count();i++) {
			UnModbusRedirectEntry e=entries.at(i);
			if (e.project!=PROJ) continue; // this line is not for us...
			string drvSection="mod_"+e.driver;
			// check if we have entries for this driver in the config file.
			dyn_string valueList;
			rc=paCfgReadValueList(configFilePath,drvSection,"tcpServerPort",valueList);
			// we may have errCode=46 meaning there are no values, which we ignore
			dyn_errClass err=getLastError();
			if (dynlen(err) && getErrorCode(err)!=46) return ERROR("Cannot read from project config","",err);
			// as we always remove (cleanup) and then add again to the config file
			// let us add to the newDrvPorts, regardless whether it exists or not,
			// unless some misconfiguration is detected
			if (valueList.count()>1) return ERROR("More than one tcpServerPort in proj config's ["+drvSection+"]","Please resolve manually - entries: "+strjoin(valueList,",")); 			
			newDrvPorts[drvSection]=e.port;   // this one will need to be added
		}

		// now process all the mod_ entries and see if there is one that needs to be removed
		// namely (UNTOOL-27) when the driver number was edited
		// but be aware that there may be some drivers added by other components,
		// and these usually have the port set to default - 502.
		// we will keep them as such then
		for (int i=1;i<=255;i++) {
			dyn_string valueList;
			int rc=paCfgReadValueList(configFilePath,"mod_"+i,"tcpServerPort",valueList);
			dyn_errClass err=getLastError();
			if (dynlen(err) && getErrorCode(err)!=46) return ERROR("Cannot read from project config","",err);
			if (valueList.count() > 0) {
				// means there is an entry; mark for cleanup unless it is with default port...
				unsigned uPortInFile=valueList.first();
				if (uPortInFile==502) {
					INFO("ProjConfig: Keeping the entry with default port for [mod_"+i+"]");
				} else {
					rmDrvPorts["mod_"+i]=uPortInFile;
				}
			}
		}
	

		if (newDrvPorts.count()==0 && rmDrvPorts.count()==0) {
			INFO("No change to project config file");
			return 0;
		}

		// make a backup of the config file
		string bkpFilePath=configFilePath+"_"+formatTime("%Y_%m_%d_%H_%M_%S",getCurrentTime())+"_unModbusRedirect";
		bool ok=copyFile(configFilePath,bkpFilePath,true);
		if (!ok) {
			return ERROR("Could not backup project config file",getLastError());
		}
		//INFO("ProjConfig: Backup of project config file created:"+bkpFilePath);

		bool needRestore=false;
		for (int i=0;i<rmDrvPorts.count();i++) {
			//rc=paCfgDeleteValue(configFilePath,rmDrvPorts.keyAt(i),"tcpServerPort",curDrvPorts.valueAt(i));
			rc=paCfgDeleteValue(configFilePath,rmDrvPorts.keyAt(i),"tcpServerPort");
			if (rc) {
				ERROR("Failed to cleanup cfgfile section "+rmDrvPorts.keyAt(i),getLastError());
				needRestore=true;
				break;
			}
			INFO("ProjConfig: Removed from ["+rmDrvPorts.keyAt(i)+"] tcpServerPort="+rmDrvPorts.valueAt(i));
		}

		if (!needRestore) { // only if there was no error before
			for (int i=0;i<newDrvPorts.count();i++) {
				rc=paCfgInsertValue(configFilePath,newDrvPorts.keyAt(i),"tcpServerPort",newDrvPorts.valueAt(i));
				if (rc) {
					ERROR("Failed to edit cfgfile section "+curDrvPorts.keyAt(i),getLastError());
					needRestore=true;
					break;
				}
				INFO("ProjConfig: Added to ["+newDrvPorts.keyAt(i)+"] tcpServerPort="+newDrvPorts.valueAt(i));
			}
		}

		int retVal=0;
		
		if (needRestore) {
				INFO("ProjConfig: Restoring original project config file from backup:"+bkpFilePath);
				copyFile(bkpFilePath, configFilePath);
				retVal==1;
		}
		//INFO("ProjConfig: Removing the backup of the project config file:"+bkpFilePath);
		remove (bkpFilePath);
		return retVal;
	}

	public string serialize()
	{
		string s;
		for (int i=1;i<=dynlen(entries);i++) {
			if (i!=1) s+="\n";
			s+=entries[i].serialize();
		}
		return s;
	}

	public static UnModbusRedirectConfiguration deserialize(string data)
	{
		UnModbusRedirectConfiguration cfg;
	
		dynClear(cfg.entries);

		dyn_string ds=strsplit(data,"\n");
		for (int i=1;i<=dynlen(ds);i++) {
			UnModbusRedirectEntry e=UnModbusRedirectEntry::deserialize(ds[i]);
			dynAppend(cfg.entries,e);
		}
		cfg.error="";
		return cfg;			
	}

	public int find(string plcip, int port)
	{
		// iteration using 0-based at()
		for (int i=0;i<dynlen(entries);i++) {
			UnModbusRedirectEntry e=at(i);
			if ( (e.plcIP==plcip) && (e.port==port) ) return i;
		}
		return -1; // not found
	}

};
