/**@name LIBRARY: unIEC104.ctl

@author: Jonas Arroyo (BE-ICS-FD)

Creation Date: 2019-12-02






/**
Purpose: register callback function

Parameters:
*/
void IEC104_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  //DebugTN("IEC104_WidgetRegisterCB() -> sDp = " + sDp + ", bSystemConnected = " + bSystemConnected);


  bool bRemote;
  int iAction;
  string sDeviceName, sSystemName, sFrontEnd, sSystemIntegrityDp;
  dyn_string exceptionInfo;


  g_bUnSystemAlarmFrontEnd = FALSE;

  sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if( sDeviceName == "" )
  {
    // In case of disconnection
    sDeviceName = g_sDpName;
  }

  // Get the system name
  sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

  // Get the front end name
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(sDeviceName);
  if( sFrontEnd == "" )
  {
    DebugTN("IEC104_WidgetRegisterCB() -> ERROR -> Error registering widget for device: " + sDp);
    return;
  }

  // Check if it is in a distributed system
  unDistributedControl_isRemote(bRemote, sSystemName);
  if( bRemote )
  {
    g_bSystemConnected = bSystemConnected;
  }
  else
  {
    g_bSystemConnected = TRUE;
  }

  // Get system integrity name
  sSystemIntegrityDp = sSystemName + c_unSystemAlarm_dpPattern + DS_pattern + sFrontEnd;
  if( dpExists(sSystemIntegrityDp) )
  {
    g_bUnSystemAlarmFrontEnd = TRUE;
  }


  // Get the registration state
  unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    DebugTN("IEC104_WidgetRegisterCB() -> ERROR -> Error getting the registering state for front end: " + sFrontEnd + ", due to: " + exceptionInfo[2]);
    return;
  }


  // Check the registration state
  switch( iAction )
  {
    case UN_ACTION_DISCONNECT:
      IEC104_WidgetDisconnection(g_sWidgetType);
      break;


    case UN_ACTION_DPCONNECT:
      g_sDpName = sDeviceName;
      IEC104_WidgetConnect(sDeviceName, sFrontEnd, sSystemIntegrityDp);
      break;


    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      IEC104_WidgetDisconnect(sDeviceName, sFrontEnd, sSystemIntegrityDp);
      IEC104_WidgetDisconnection(g_sWidgetType);
      break;


    case UN_ACTION_NOTHING:
    default:
      DebugTN("IEC104_WidgetRegisterCB() -> ERROR -> Error unknown registration state: " + iAction + ", for the front end: " + sFrontEnd);
      return;
      break;
  }

}//  IEC104_WidgetRegisterCB()





void IEC104_WidgetConnect(string sDeviceName, string sFrontEnd, string sSystemIntegrityDp)
{
  int iRes;
  string sSystemName;
  dyn_string dsColor;


  //DebugTN("IEC104_WidgetConnect() -> sDeviceName = " + sDeviceName + ", sFrontEnd = " + sFrontEnd + ", sSystemIntegrityDp = " + sSystemIntegrityDp);

  // Check if the device name is of type "SOFT_FE" if not like of no PLC. -> not available.
  if( dpTypeName(sDeviceName) != "IEC104" )
  {
    g_bUnSystemAlarmFrontEnd = FALSE;
  }

  sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
  if( g_bUnSystemAlarmFrontEnd )
  {
    unGenericObject_getFrontEndColorState(makeDynString(sSystemIntegrityDp + ".alarm"), dsColor);
    g_sCommColor = dsColor[1];
    iRes         = dpConnect("IEC104_WidgetCB", sSystemIntegrityDp + ".alarm",
                                                sSystemIntegrityDp + ".enabled");

    g_bCallbackConnected = (iRes >= 0);
  }
  else
  {
    IEC104_WidgetDisconnection(g_sWidgetType);
  }

}//  IEC104_WidgetConnect()





/**
Purpose: widget animation function

Parameters:

*/
void IEC104_WidgetCB(string sDpSystemIntegrityAlarmValue,   int  iFrontEndSystemIntegrityAlarmValue,
                     string sDpSystemIntegrityAlarmEnabled, bool bFrontEndSystemIntegrityAlarmEnabled)
{
  string sWarningLetter, sWarningColor, sBodyColor, sText, sCompleteText;


  //DebugTN("IEC104_WidgetCB() -> iFrontEndSystemIntegrityAlarmValue = " + iFrontEndSystemIntegrityAlarmValue + ", bFrontEndSystemIntegrityAlarmEnabled = ", bFrontEndSystemIntegrityAlarmEnabled);

  sBodyColor = "white";
  sText      = "Not initialized";

  if( g_bSystemConnected )
  {
    unGenericObject_getFrontState(makeDynString("COM ERR"),
                                  makeDynString(g_sCommColor),
                                  makeDynInt(iFrontEndSystemIntegrityAlarmValue),
                                  iFrontEndSystemIntegrityAlarmValue,
                                  bFrontEndSystemIntegrityAlarmEnabled,
                                  sText,
                                  sBodyColor,
                                  sWarningLetter,
                                  sWarningColor,
                                  sCompleteText);

    setMultiValue("WidgetArea",  "visible", TRUE,
                  "WarningText", "text",    sWarningLetter,
                  "WarningText", "foreCol", sWarningColor,
                  "Body1",       "foreCol", sBodyColor,
                  "Body1",       "text",    sText);
  }

}//  IEC104_WidgetCB();





/**
Purpose: widget disconnect function

Parameters:

Constraints:
  . Global variables defined in Analog faceplate
*/

void IEC104_WidgetDisconnect(string sDeviceName, string sFrontEnd, string sSystemIntegrityDp)
{
  int iRes;
  string sSystemName;


  //DebugTN("IEC104_WidgetDisconnect() -> sDeviceName = " + sDeviceName + ", sFrontEnd = " + sFrontEnd + ", sSystemIntegrityDp = ", sSystemIntegrityDp);

  sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

  if( g_bUnSystemAlarmFrontEnd)
  {
    iRes = dpDisconnect("IEC104_WidgetCB", sSystemIntegrityDp + ".alarm",
                                           sSystemIntegrityDp + ".enabled");
    if( iRes < 0 )
    {
      g_bCallbackConnected = TRUE;
    }
    else
    {
      g_bCallbackConnected = FALSE;
    }

  }

}//  IEC104_WidgetDisconnect()





void IEC104_WidgetDisconnection(string sWidgetType)
{
  //DebugTN("IEC104_WidgetDisconnection() -> sWidgetType = ", sWidgetType);


  setMultiValue("Body1",       "foreCol", "unDataNoAccess",
                "Body1",       "text",    "Not initialized",
                "WidgetArea",  "visible", FALSE,
                "WarningText", "text",    "");


}//  IEC104_WidgetDisconnection()





/**
Purpose: configuration of popup menu

Parameters:
  - sDpName, string, input, device name
  - sDpType, string, input, unicos object
  - dsAccessOk, dyn_string, input, authorized action
  - dsMenuList, dyn_string, output, menu to pop-up
*/
void IEC104_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)
{

  _UnPlc_MenuConfiguration(sDpName, sDpType, dsAccessOk, dsMenuList);

}//  IEC104_MenuConfiguration()





/**
Purpose: handle the answer of the popup menu

Parameters:
  - deviceName, string, input, the device dp name
  - sDpType, string, input, the device dp type
  - dsMenuList, dyn_string, input, list of requested action
  - iMenuAnswer, int, input, selected action from the menuList
*/
void IEC104_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
{

  _UnPlc_HandleMenu(sDeviceName, sDpType, dsMenuList, iMenuAnswer);

}//  IEC104_HandleMenu()





/**
Purpose: faceplate register callback function

Parameters:
*/
void IEC104_FaceplateRegisterCB(string sDp, bool bSystemConnected)
{

  _UnPlc_FaceplateRegisterCB(sDp, bSystemConnected);

}//  IEC104_FaceplateRegisterCB()





/**
Purpose: Animate Faceplate disconnection

Parameters:
*/
void IEC104_FaceplateDisconnection()
{

  _UnPlc_FaceplateDisconnection();

}// IEC104_FaceplateDisconnection()






/**
Purpose: Function called from the objectList to return the time and value

Parameters:
  - sDeviceName, string, input, device name
  - sDeviceType, string, input, unicos object
  - dsReturnData, dyn_string, output, return value
*/

IEC104_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
  bool bEnable;
  int iCom;
  string sTime, sSystemName, sFrontEnd, sWarningLetter, sWarningColor, sBodyColor, sText, sCompleteText;
  dyn_string dsColor;


  sBodyColor = "unDataNoAccess";
  sText      = "???";
  sTime      = "0";


  // get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

  // get the PLC name
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(sDeviceName);
  if( dpExists(sSystemName + c_unSystemAlarm_dpPattern + DS_pattern + sFrontEnd + ".alarm") )
  {
    dpGet(sSystemName + c_unSystemAlarm_dpPattern + DS_pattern  +sFrontEnd + ".alarm",                 iCom,
          sSystemName + c_unSystemAlarm_dpPattern + DS_pattern + sFrontEnd + ".enabled",               bEnable,
          sSystemName + c_unSystemAlarm_dpPattern + DS_pattern + sFrontEnd + ".alarm:_online.._stime", sTime);

    unGenericObject_getFrontEndColorState(makeDynString(sSystemName + c_unSystemAlarm_dpPattern + DS_pattern + sFrontEnd + ".alarm"),
                                          dsColor);
    unGenericObject_getFrontState(makeDynString("COM ERR"),
                                  makeDynString(dsColor[1]),
                                  makeDynInt(iCom),
                                  iCom,
                                  bEnable,
                                  sText,
                                  sBodyColor,
                                  sWarningLetter,
                                  sWarningColor,
                                  sCompleteText);
  }

  dsReturnData[1] = sTime;
  dsReturnData[2] = sText;
  dsReturnData[3] = "FALSE";
  dsReturnData[4] = sCompleteText;
  dsReturnData[5] = sBodyColor;

}
